import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.io.TempDir;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrowsExactly;

@DisplayName("ReadFile class functions test")
class ReadFileTest {

	@TempDir
	static Path tempDir;

	@Nested
	@DisplayName("Testing ReadFile `open()` function")
	class OpenTest {
		
		@Test
		@DisplayName("Empty file name in `filename` parameter")
		void emptyFileName() {
			assertThrowsExactly(IllegalArgumentException.class, () -> ReadFile.open(Path.of("")));
		}

		@Test
		@DisplayName("Empty file name in `filename` parameter")
		void invalidFileExtension() throws IOException {
			// Create files with wrong file extensions
			Path csv = Files.createFile(tempDir.resolve("emails.csv"));
			Path empty = Files.createFile(tempDir.resolve("emails"));
			Path xlsx = Files.createFile(tempDir.resolve("emails.xlsx"));
			Path db = Files.createFile(tempDir.resolve("emails.db"));

			assertThrowsExactly(IllegalArgumentException.class, () -> ReadFile.open(csv));
			assertThrowsExactly(IllegalArgumentException.class, () -> ReadFile.open(empty));
			assertThrowsExactly(IllegalArgumentException.class, () -> ReadFile.open(xlsx));
			assertThrowsExactly(IllegalArgumentException.class, () -> ReadFile.open(db));
		}
		
		@Test
		@DisplayName("Trying to open non-existent file")
		void nonExistentFile() throws IOException {
			Path nonExistent = Files.createFile(tempDir.resolve("doesNotExist.txt"));
			Files.delete(nonExistent);
			
			assertThrowsExactly(RuntimeException.class, () -> ReadFile.open(nonExistent));
		}
	}
}